import React from "react";
import { AreaChart, Tooltip, Area, ResponsiveContainer } from "recharts";

type DataType = {
  real?: number[];
  attempts?: number[];
  title: string;
};

const Charts = ({ real, attempts, title }: DataType) => {
  if (!real || !attempts) return null;

  const data = real.map((_item, index) => ({
    actual: _item,
    attempt: attempts[index],
  }));

  return (
    <div className="mt-5">
      <h4>{title}</h4>
      <ResponsiveContainer height={280} width="100%">
        <AreaChart data={data}>
          <Tooltip />
          <Area
            type="monotone"
            dataKey="actual"
            stroke="red"
            fillOpacity={0.2}
            fill="red"
          />
          <Area
            type="monotone"
            dataKey="attempt"
            stroke="blue"
            fillOpacity={0.2}
            fill="blue"
          />
        </AreaChart>
      </ResponsiveContainer>
    </div>
  );
};

export default Charts;
