import React, { useState, useRef, useEffect } from "react";
import { initialiseControls } from "../util/controls";
import { keyEvent } from "../util/keystrokeLogger";
import { KeyDetails, Result } from "../util/types";
import Charts from "./Charts";
import ResultTable from "./ResultTable";

const controls = {
  standard: {
    sd: 1.5,
    threshold: 65,
    use: true,
  },
  fullStandard: {
    threshold: 1,
    use: true,
  },
};

const Login = () => {
  const [credentials, setCredentials] = useState({
    userName: "",
    password: "",
  });

  const [result, setResult] = useState<Result | null>(null);

  const keyCount = useRef<number>(0);
  const keydownArray = useRef<KeyDetails[]>([]);
  const keyupArray = useRef<KeyDetails[]>([]);

  const onChangeCredentials = (e: React.ChangeEvent<HTMLInputElement>) => {
    setCredentials({ ...credentials, [e.target.name]: e.target.value });
  };

  useEffect(() => {
    initialiseControls(controls);
  }, []);

  const clearData = () => {
    setCredentials({ ...credentials, password: "" });
    keyCount.current = 0;
    keydownArray.current = [];
    keyupArray.current = [];
  };

  const submit = () => {
    const data = {
      username: credentials.userName,
      password: credentials.password,
      keydown: keydownArray.current,
      keyup: keyupArray.current,
      controls,
    };
    clearData();

    fetch("http://localhost:3001/user/login", {
      method: "POST",
      mode: "cors",
      headers: {
        "Content-Type": "application/json",
      },
      body: JSON.stringify(data),
    }).then(async (res) => {
      let response: Result = await res.json();
      if (res.ok) {
        setResult(response);
      } else {
        console.log(response);
      }
    });
  };

  const onKeyDown = (e: any) => {
    const details = keyEvent(e);
    if (!details) return;

    if (
      ["Backspace"].includes(details.code) ||
      ["Backspace"].includes(details.code)
    ) {
      return clearData();
    }

    keydownArray.current.push(details);
    keyupArray.current.push({ ...details, time: null });
    keyCount.current++;
  };

  const onKeyUp = (e: any) => {
    const details = keyEvent(e);
    if (!details) return;
    if (!details.time) details.time = Date.now();

    if (
      ["Backspace"].includes(details.code) ||
      ["Backspace"].includes(details.code)
    ) {
      return clearData();
    }

    var reqdUpKeystroke = keyupArray.current.find(
      (element) => element.code === details.code && !element.time
    );

    if (reqdUpKeystroke) reqdUpKeystroke.time = details.time;

    if (details.code === "Enter") submit();
  };

  const disable = credentials.userName === "" || credentials.password === "";

  return (
    <div className="container">
      <div className="row mt-5">
        <div className="col-lg-4">
          <div className="card card-body pt-4 pb-4">
            <h5 className="card-title">Login</h5>
            <div className="mb-3">
              <label htmlFor="exampleFormControlInput1" className="form-label">
                User name
              </label>
              <input
                className="form-control"
                type="text"
                aria-label=".form-control-lg example"
                name="userName"
                onChange={onChangeCredentials}
              />
            </div>

            <div className="mb-3">
              <label htmlFor="exampleFormControlInput1" className="form-label">
                Password
              </label>
              <input
                className="form-control"
                type="password"
                aria-label=".form-control-lg example"
                name="password"
                id="password"
                onChange={onChangeCredentials}
                onKeyDown={onKeyDown}
                onKeyUp={onKeyUp}
              />
            </div>
            <button
              type="button"
              className="btn btn-primary"
              onClick={submit}
              disabled={disable}
            >
              Login
            </button>
          </div>
        </div>
        <div className="col-lg">
          <ResultTable data={result?.result} />
        </div>
      </div>
      <div className="row  mt-5 ">
        <Charts
          title="Overall keystroke"
          real={result?.db.full}
          attempts={result?.attempt.full}
        />
      </div>
      <div className="row">
        <Charts
          title="DD keystroke"
          real={result?.db.dd}
          attempts={result?.attempt.dd}
        />
        <Charts
          title="Hold keystroke"
          real={result?.db.hold}
          attempts={result?.attempt.hold}
        />
        <Charts
          title="Flight keystroke"
          real={result?.db.flight}
          attempts={result?.attempt.flight}
        />
      </div>
    </div>
  );
};

export default Login;
